package tis4.auth.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Scanner;

import tis4.auth.client.gui.LoginScreen;
import tis4.auth.client.gui.MainScreen;

public class ClientMain{
	private String host;
	private int port;
	private Socket socket;
	private PrintWriter output;
	private Scanner input;
	private ClientListener listener;
	private LoginScreen login;
	private MainScreen mainScreen = null;
	public static void main(String[] args) {
		String host = null;
		int port = 0;
		if(args.length>1){
			host = args[0];
			try{
				port = Integer.parseInt(args[1]);
			}catch(NumberFormatException e){
				System.exit(0);
			}
		}else{
			host = "localhost";
			port = 4444;
		}
		new ClientMain(host,port);
		
	}
	public ClientMain(String host, int port){
		this.host = host;
		this.port = port;
		login = new LoginScreen(this);
	}
	public void connect() throws IOException{
		if(socket==null||!socket.isConnected()){
			System.out.println("Conneting...");
			socket = new Socket(host,port);
			input = new Scanner(new BufferedReader(new InputStreamReader(socket.getInputStream())));
			output = new PrintWriter(socket.getOutputStream(),true);
			if(listener!=null){
				listener.stop();
				listener = null;
			}
			listener = new ClientListener(this);
			Thread t = new Thread(listener);
			t.start();
		}
	}
	public PrintWriter getOutput(){
		return output;
	}
	public void login(String username, String password){
		if(username!=null&&password!=null){
			output.println("<login><user>"+username.trim()+"</user><pass>"+password.trim()+"</pass></login>");
		}else{
			login.setLoggedIn(false);
		}
	}
	public void exit(){
		close();
		System.exit(0);
	}
	public void close(){
		listener.stop();
		listener = null;
		output.println("<admin><command>disconnect</command></admin>");
		if(!socket.isClosed()){
			try {
				socket.close();
			} catch (IOException e) {
				System.err.println("Could not close client socket");
				e.printStackTrace();
			}
		}
		socket = null;
		input = null;
		output = null;
		System.out.println("Socket disconnected");
	}
	public LoginScreen getLoginScreen(){
		return login;
	}
	public void setMainScreen(MainScreen m){
		mainScreen = m;
	}
	public MainScreen getMainScreen(){
		return mainScreen;
	}
	public Scanner getInput(){
		return input;
	}

}
