package tis4.auth.client;

import java.util.ArrayList;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import tis4.auth.client.XMLElement;
import tis4.auth.client.valueparser.Admin;
import tis4.auth.client.valueparser.Login;
import tis4.auth.client.valueparser.ValueParser;
/**
 * 
 * @author tim
 * This class handles all of the incoming xml, checks the root element and if we have something useful there and if we do, 
 * it creates name value pairs of the remaining components and passes them to the ValueParser
 */
public class XMLHandler extends DefaultHandler{
	private ClientMain clientSocket;
	private ArrayList<XMLElement> valueList;
	public XMLHandler(ClientMain clientSocket){
		this.clientSocket = clientSocket;
		valueList = new ArrayList<XMLElement>();
	}
	/**
	 * This is called by the parser when an element begins
	 * if there is no root element setup we try to set one up
	 */
	public void startElement(String uri, String localName,String qName,Attributes attributes) throws SAXException {
		valueList.add(new XMLElement(qName,TagType.START));
	}
	/**
	 * called by the parser when an element ends 
	 * if theres a value pair setup add it to the list to pass on and if not and its the current action ending pass the list to that place
	 */
	public void endElement(String uri, String localName,String qName) throws SAXException {
		XMLElement element = findElement(qName);
		if(element!=null){
			if(element.getValue()!=null&&element.getType()==TagType.VALUE){
				element.setType(TagType.FULL);
			}else if(element.getType()==TagType.START){
				valueList.add(new XMLElement(qName, TagType.END));
			}
		}
	}
	/**
	 * Called by the parser, sets the value of the current name value pair if applicable
	 */
	public void characters(char ch[], int start, int length) throws SAXException {
		String input = new String(ch,start,length);
		XMLElement latest = valueList.get(valueList.size()-1);
		if(latest!=null&&latest.getType()==TagType.START){
			latest.setValue(input);
			latest.setType(TagType.VALUE);
		}
	}
	/**
	 * 
	 * @param name
	 * @return an xmlelement based on the input name inside the value list
	 */
	private XMLElement findElement(String name){
		for(XMLElement e: valueList){
			if(e.getName().equals(name)){
				return e;
			}
		}
		return null;
	}
	/**
	 * Checks the first element of the value list to find out where to send the thing and sends it on its merry way whilst resetting the system
	 */
	public void passOn(){
		ValueParser currentAction = null;
		if(valueList!=null&&valueList.size()>0){
			String rootElement = valueList.get(0).getName();
			if("login".equalsIgnoreCase(rootElement)){
				currentAction = new Login(clientSocket);
			}else if("admin".equalsIgnoreCase(rootElement)){
				currentAction = new Admin(clientSocket);
			}
		}
		if(currentAction!=null){
			currentAction.parse(valueList);
		}
		valueList = null;
		valueList = new ArrayList<XMLElement>();
	}
}
