package tis4.auth.server;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.net.Socket;
import java.util.Scanner;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class ClientSocket implements Runnable{
	private static final long SLEEP_TIME = 10; //sleep time milliseconds
	private Socket clientSocket;
	private ServerMain serverMain;
	private Scanner input;
	private PrintWriter output;
	private boolean run = true;
	private SAXParser saxParser = null;
	private XMLHandler xmlHandler = null;
	private boolean authenticated = false;
	public ClientSocket(Socket s, ServerMain serverMain){
		this.clientSocket = s;
		this.serverMain = serverMain;
		//setup input reader
		try {
			input = new Scanner(new BufferedInputStream(clientSocket.getInputStream()));
		} catch (IOException e) {
			System.err.println("Could not acquire client input stream");
			e.printStackTrace();
		}
		
		//setup output
		try {
			output = new PrintWriter(clientSocket.getOutputStream(),true);
		} catch (IOException e) {
			System.err.println("Could not acquire output stream");
			e.printStackTrace();
		}
		
		System.out.println("Setting up XML Parse");
		SAXParserFactory factory = SAXParserFactory.newInstance();
		try {
			saxParser = factory.newSAXParser();
		} catch (ParserConfigurationException e) {
			close();
		} catch (SAXException e) {
			close();
		}
		xmlHandler = new XMLHandler(this);
		System.out.println("Client connection "+clientSocket.getInetAddress());
	}
	/**
	 * Started the main loop which listens for incoming data and passes it on if it exists
	 */
	@Override
	public void run() {
		while(run){
			while(input.hasNextLine()){
				String line = input.nextLine();
				parseInput(line);
				Thread.yield(); //just to make sure we're not hogging cpu during a large receive
			}
			//we need to sleep to make sure we don't max out execution
			try {
				Thread.sleep(SLEEP_TIME);
			} catch (InterruptedException e) {
				System.err.println("Could not sleep client thread");
			}
		}
	}
	/**
	 * Where incoming data is passed to
	 * @param line
	 * We trim it just to make sure there's nothing unwanted living in there then pass the string to the xml parser
	 */
	public void parseInput(String line){
		line = line.trim();
		try {
			saxParser.parse(new InputSource(new StringReader(line)),xmlHandler);
			xmlHandler.passOn();
		} catch (SAXException e) {
			System.err.println("Could not parse XML");
		} catch (IOException e) {
			System.err.println("Could not parse XML");
		}
	}
	/**
	 * Closes client socket with reason
	 * @param reason
	 */
	public void close(){
		System.out.println("Client disconnecting: "+clientSocket.getInetAddress());
		run = false;
		output.println("<admin><command>disconnect</command></admin>");
		try {
			clientSocket.close();
		} catch (IOException e) {
			System.err.println("Could not close client socket");
			e.printStackTrace();
		}
	}
	/**
	 * Checks authentication (passed on up to the main class) and disconnects on failure or greets if success
	 * @param user
	 * @param pass
	 */
	public void authenticate(String user, String pass){
		authenticated = serverMain.authenticate(user, pass);
		if(!authenticated){
			output.println("<login><error>incorrect credential</error></login>");
			close();
		}else{
			output.println("<login><info>login success</info></login>");
		}
	}
	public PrintWriter getWriter(){
		return output;
	}
	public boolean isLoggedIn(){
		return authenticated;
	}
}
